@@ -64,6 +64,6 @@ bool ppc_elf_inline_plt (struct bfd_link_info *);
 asection *ppc_elf_tls_setup (bfd *, struct bfd_link_info *);
 bool ppc_elf_tls_optimize (bfd *, struct bfd_link_info *);
 void ppc_elf_maybe_strip_sdata_syms (struct bfd_link_info *);
-extern bool ppc_elf_modify_segment_map (bfd *, struct bfd_link_info *);
-extern bool ppc_elf_section_processing (bfd *, Elf_Internal_Shdr *);
+extern bool ppc_elf_modify_segment_map (bfd *, struct bfd_link_info *)
+  ATTRIBUTE_HIDDEN;
 extern bool ppc_finish_symbols (struct bfd_link_info *);
