@@ -378,49 +378,21 @@ verilog_mkobject (bfd *abfd)
   return true;
 }
 
-#define verilog_close_and_cleanup		     _bfd_generic_close_and_cleanup
-#define verilog_bfd_free_cached_info		     _bfd_generic_bfd_free_cached_info
-#define verilog_new_section_hook		     _bfd_generic_new_section_hook
-#define verilog_bfd_is_target_special_symbol	     _bfd_bool_bfd_asymbol_false
-#define verilog_bfd_is_local_label_name		     bfd_generic_is_local_label_name
-#define verilog_get_lineno			     _bfd_nosymbols_get_lineno
-#define verilog_find_nearest_line		     _bfd_nosymbols_find_nearest_line
-#define verilog_find_nearest_line_with_alt	     _bfd_nosymbols_find_nearest_line_with_alt
-#define verilog_find_inliner_info		     _bfd_nosymbols_find_inliner_info
-#define verilog_make_empty_symbol		     _bfd_generic_make_empty_symbol
-#define verilog_bfd_make_debug_symbol		     _bfd_nosymbols_bfd_make_debug_symbol
-#define verilog_read_minisymbols		     _bfd_generic_read_minisymbols
-#define verilog_minisymbol_to_symbol		     _bfd_generic_minisymbol_to_symbol
-#define verilog_bfd_get_relocated_section_contents   bfd_generic_get_relocated_section_contents
-#define verilog_bfd_relax_section		     bfd_generic_relax_section
-#define verilog_bfd_gc_sections			     bfd_generic_gc_sections
-#define verilog_bfd_merge_sections		     bfd_generic_merge_sections
-#define verilog_bfd_is_group_section		     bfd_generic_is_group_section
-#define verilog_bfd_group_name			     bfd_generic_group_name
-#define verilog_bfd_discard_group		     bfd_generic_discard_group
-#define verilog_section_already_linked		     _bfd_generic_section_already_linked
-#define verilog_bfd_link_hash_table_create	     _bfd_generic_link_hash_table_create
-#define verilog_bfd_link_add_symbols		     _bfd_generic_link_add_symbols
-#define verilog_bfd_link_just_syms		     _bfd_generic_link_just_syms
-#define verilog_bfd_final_link			     _bfd_generic_final_link
-#define verilog_bfd_link_split_section		     _bfd_generic_link_split_section
-
 const bfd_target verilog_vec =
 {
   "verilog",			/* Name.  */
   bfd_target_verilog_flavour,
   BFD_ENDIAN_UNKNOWN,		/* Target byte order.  */
   BFD_ENDIAN_UNKNOWN,		/* Target headers byte order.  */
-  (HAS_RELOC | EXEC_P |		/* Object flags.  */
-   HAS_LINENO | HAS_DEBUG |
-   HAS_SYMS | HAS_LOCALS | WP_TEXT | D_PAGED),
+  EXEC_P,			/* Object flags.  */
   (SEC_CODE | SEC_DATA | SEC_ROM | SEC_HAS_CONTENTS
-   | SEC_ALLOC | SEC_LOAD | SEC_RELOC),	/* Section flags.  */
+   | SEC_ALLOC | SEC_LOAD),	/* Section flags.  */
   0,				/* Leading underscore.  */
   ' ',				/* AR_pad_char.  */
   16,				/* AR_max_namelen.  */
   0,				/* match priority.  */
   TARGET_KEEP_UNUSED_SECTION_SYMBOLS, /* keep unused section symbols.  */
+  false,			/* merge sections */
   bfd_getb64, bfd_getb_signed_64, bfd_putb64,
   bfd_getb32, bfd_getb_signed_32, bfd_putb32,
   bfd_getb16, bfd_getb_signed_16, bfd_putb16,	/* Data.  */
