@@ -34,7 +34,8 @@
 #define SFRAME_FRE_ELEM_LOC_REG		0
 #define SFRAME_FRE_ELEM_LOC_STACK	1
 
-#define SFRAME_FRE_BASE_REG_INVAL	((unsigned int)-1)
+/* An invalid register number.  */
+#define SFRAME_FRE_REG_INVALID		((unsigned int)-1)
 
 /* SFrame Frame Row Entry (FRE).
 
