@@ -1,6 +1,6 @@
 // dynobj.cc -- dynamic object support for gold
 
-// Copyright (C) 2006-2022 Free Software Foundation, Inc.
+// Copyright (C) 2006-2025 Free Software Foundation, Inc.
 // Written by Ian Lance Taylor <iant@google.com>.
 
 // This file is part of gold.
@@ -1756,12 +1756,7 @@ Versions::symbol_section_contents(const Symbol_table* symtab,
       unsigned int version_index;
       const char* version = (*p)->version();
       if (version == NULL)
-	{
-	  if ((*p)->is_defined() && !(*p)->is_from_dynobj())
-	    version_index = elfcpp::VER_NDX_GLOBAL;
-	  else
-	    version_index = elfcpp::VER_NDX_LOCAL;
-	}
+	version_index = elfcpp::VER_NDX_LOCAL;
       else if (version[0] == '\0')
         version_index = elfcpp::VER_NDX_GLOBAL;
       else
