@@ -1,6 +1,6 @@
 // target-reloc.h -- target specific relocation support  -*- C++ -*-
 
-// Copyright (C) 2006-2020 Free Software Foundation, Inc.
+// Copyright (C) 2006-2025 Free Software Foundation, Inc.
 // Written by Ian Lance Taylor <iant@google.com>.
 
 // This file is part of gold.
@@ -136,8 +136,7 @@ class Default_comdat_behavior
     if (Layout::is_debug_info_section(name))
       return CB_PRETEND;
     if (strcmp(name, ".eh_frame") == 0
-#define ATTR_SECTION_PREFIX ".gnu.build.attributes"
-	|| strncmp(name, ATTR_SECTION_PREFIX, sizeof (ATTR_SECTION_PREFIX) - 1) == 0
+	|| is_prefix_of (".gnu.build.attributes", name)
 	|| strcmp(name, ".gcc_except_table") == 0)
       return CB_IGNORE;
     return CB_ERROR;
@@ -243,7 +242,7 @@ issue_discarded_error(
 	  relinfo, shndx, offset,
 	  _("relocation refers to local symbol \"%s\" [%u], "
 	    "which is defined in a discarded section"),
-	  object->get_symbol_name(r_sym), r_sym);
+	  object->get_symbol_name(r_sym).c_str(), r_sym);
     }
   else
     {
@@ -265,7 +264,7 @@ issue_discarded_error(
 							  &key_symndx);
       if (key_symndx != 0)
 	gold_info(_("  section group signature: \"%s\""),
-		  object->get_symbol_name(key_symndx));
+		  object->get_symbol_name(key_symndx).c_str());
       if (kept_obj != NULL)
 	gold_info(_("  prevailing definition is from %s"),
 		  kept_obj->name().c_str());
