@@ -1,5 +1,5 @@
 /* Binary mode I/O.
-   Copyright (C) 2001-2023 Free Software Foundation, Inc.
+   Copyright (C) 2001-2025 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@
 # undef O_BINARY
 # undef O_TEXT
 #endif
-#if O_BINARY
+#if defined(O_BINARY) && O_BINARY
 # if defined __EMX__ || defined __DJGPP__ || defined __CYGWIN__
 #  include <io.h> /* declares setmode() */
 # else
