@@ -1,6 +1,6 @@
 /* LoongArch assembler/disassembler support.
 
-   Copyright (C) 2021-2024 Free Software Foundation, Inc.
+   Copyright (C) 2021-2025 Free Software Foundation, Inc.
    Contributed by Loongson Ltd.
 
    This file is part of GNU Binutils.
@@ -36,6 +36,7 @@ extern "C"
 
   #define LARCH_MK_ADDI_D 0xffc00000
   #define LARCH_OP_ADDI_D 0x02c00000
+  #define LARCH_OP_ADDI_W 0x02800000
   #define LARCH_MK_PCADDI 0xfe000000
   #define LARCH_OP_PCADDI 0x18000000
   #define LARCH_MK_B 0xfc000000
@@ -44,6 +45,7 @@ extern "C"
   #define LARCH_OP_BL 0x54000000
   #define LARCH_MK_ORI 0xffc00000
   #define LARCH_OP_ORI 0x03800000
+  #define LARCH_OP_OR 0x00150000
   #define LARCH_MK_LU12I_W 0xfe000000
   #define LARCH_OP_LU12I_W 0x14000000
   #define LARCH_MK_LD_D 0xffc00000
@@ -267,6 +269,10 @@ dec2 : [1-9][0-9]?
   extern const char *const loongarch_cr_normal_name[4];
   extern const char *const loongarch_v_normal_name[32];
   extern const char *const loongarch_x_normal_name[32];
+  extern const char *const loongarch_r_cfi_name[32];
+  extern const char *const loongarch_r_cfi_name_alias[32];
+  extern const char *const loongarch_f_cfi_name[32];
+  extern const char *const loongarch_f_cfi_name_alias[32];
 
   extern struct loongarch_ase loongarch_ASEs[];
 
