@@ -103,6 +103,51 @@ sframe_ret_set_errno (int *errp, int error)
   return NULL;
 }
 
+/* Allocate space for NUM_FDES number of SFrame FDEs of type
+   sframe_func_desc_entry.  This is version-unaware because this pertains to
+   libsframe's internal in-memory representation of SFrame FDE.  */
+
+static int
+sframe_fde_tbl_alloc (sf_fde_tbl **fde_tbl, unsigned int num_fdes)
+{
+  size_t fidx_size = num_fdes * sizeof (sframe_func_desc_entry);
+  size_t fd_tbl_sz = (sizeof (sf_fde_tbl) + fidx_size);
+
+  *fde_tbl = malloc (fd_tbl_sz);
+  if (*fde_tbl == NULL)
+    return SFRAME_ERR;
+
+  (*fde_tbl)->alloced = num_fdes;
+
+  return 0;
+}
+
+/* Initialize libsframe's internal representation of SFrame FDEs.  */
+
+static int
+sframe_fde_tbl_init (sf_fde_tbl *fde_tbl, const char *fde_buf,
+		     size_t *fidx_size, unsigned int num_fdes, uint8_t ver)
+{
+  /* sframe_func_desc_entry is the same type as the latest SFrame FDE V2
+     definition (currently sframe_func_desc_entry_v2).  */
+  if (ver == SFRAME_VERSION_2 && SFRAME_VERSION == SFRAME_VERSION_2)
+    {
+      *fidx_size = num_fdes * sizeof (sframe_func_desc_entry_v2);
+      memcpy (fde_tbl->entry, fde_buf, *fidx_size);
+      fde_tbl->count = num_fdes;
+    }
+  /* If ver is not the latest, read buffer manually and upgrade from
+     sframe_func_desc_entry_v2 to populate the sf_fde_tbl entries.  */
+  else
+    {
+      /* Not possible ATM.  */
+      *fidx_size = 0;
+      return SFRAME_ERR;
+    }
+
+  return 0;
+}
+
 /* Get the SFrame header size.  */
 
 static uint32_t
@@ -181,30 +226,67 @@ need_swapping (int endian)
   return 0;
 }
 
-/* Flip the endianness of the SFrame header.  */
+/* Flip the endianness of the SFrame header starting at BUF.
+   VER is the version of the SFrame data in the buffer.
 
-static void
-flip_header (sframe_header *sfheader)
+   Returns SFRAME_ERR if any error.  If error code is returned, the flipped
+   header should not be used.  */
+
+static int
+flip_header (char *buf, uint8_t ver ATTRIBUTE_UNUSED)
 {
-  swap_thing (sfheader->sfh_preamble.sfp_magic);
-  swap_thing (sfheader->sfh_preamble.sfp_version);
-  swap_thing (sfheader->sfh_preamble.sfp_flags);
-  swap_thing (sfheader->sfh_cfa_fixed_fp_offset);
-  swap_thing (sfheader->sfh_cfa_fixed_ra_offset);
-  swap_thing (sfheader->sfh_num_fdes);
-  swap_thing (sfheader->sfh_num_fres);
-  swap_thing (sfheader->sfh_fre_len);
-  swap_thing (sfheader->sfh_fdeoff);
-  swap_thing (sfheader->sfh_freoff);
+  /* SFrame header binary format has remained the same in SFRAME_VERSION_1,
+     SFRAME_VERSION_2.  */
+  sframe_header *sfh = (sframe_header *) buf;
+  swap_thing (sfh->sfh_preamble.sfp_magic);
+  swap_thing (sfh->sfh_preamble.sfp_version);
+  swap_thing (sfh->sfh_preamble.sfp_flags);
+  swap_thing (sfh->sfh_abi_arch);
+  swap_thing (sfh->sfh_cfa_fixed_fp_offset);
+  swap_thing (sfh->sfh_cfa_fixed_ra_offset);
+  swap_thing (sfh->sfh_auxhdr_len);
+  swap_thing (sfh->sfh_num_fdes);
+  swap_thing (sfh->sfh_num_fres);
+  swap_thing (sfh->sfh_fre_len);
+  swap_thing (sfh->sfh_fdeoff);
+  swap_thing (sfh->sfh_freoff);
+
+  /* Alert for missing functionatlity.  Auxiliary header, if present, needs to
+     flipped based on per abi/arch semantics.  */
+  if (sfh->sfh_auxhdr_len)
+    return SFRAME_ERR;
+
+  return 0;
 }
 
-static void
-flip_fde (sframe_func_desc_entry *fdep)
+/* Endian flip the SFrame FDE at BUF (buffer size provided in BUF_SIZE), given
+   the SFrame version VER.  Update the FDE_SIZE to the size of the SFrame FDE
+   flipped.
+
+   Return SFRAME_ERR if any error.  If error code is returned, the flipped FDEP
+   should not be used.  */
+
+static int
+flip_fde (char *buf, size_t buf_size, uint8_t ver, size_t *fde_size)
 {
-  swap_thing (fdep->sfde_func_start_address);
-  swap_thing (fdep->sfde_func_size);
-  swap_thing (fdep->sfde_func_start_fre_off);
-  swap_thing (fdep->sfde_func_num_fres);
+
+  if (ver == SFRAME_VERSION_2)
+    {
+      if (buf_size < sizeof (sframe_func_desc_entry_v2))
+	return SFRAME_ERR;
+
+      sframe_func_desc_entry_v2 *fdep = (sframe_func_desc_entry_v2 *) buf;
+      swap_thing (fdep->sfde_func_start_address);
+      swap_thing (fdep->sfde_func_size);
+      swap_thing (fdep->sfde_func_start_fre_off);
+      swap_thing (fdep->sfde_func_num_fres);
+
+      *fde_size = sizeof (sframe_func_desc_entry_v2);
+    }
+  else
+    return SFRAME_ERR; /* No other versions are possible ATM.  */
+
+  return 0;
 }
 
 /* Check if SFrame header has valid data.  */
@@ -363,10 +445,11 @@ sframe_decoder_get_funcdesc_at_index (sframe_decoder_ctx *ctx,
   num_fdes = sframe_decoder_get_num_fidx (ctx);
   if (num_fdes == 0
       || func_idx >= num_fdes
-      || ctx->sfd_funcdesc == NULL)
+      || ctx->sfd_funcdesc == NULL
+      || ctx->sfd_funcdesc->count <= func_idx)
     return sframe_ret_set_errno (&err, SFRAME_ERR_DCTX_INVAL);
 
-  fdep = &ctx->sfd_funcdesc[func_idx];
+  fdep = &ctx->sfd_funcdesc->entry[func_idx];
   return fdep;
 }
 
@@ -388,7 +471,8 @@ sframe_decoder_get_secrel_func_start_addr (sframe_decoder_ctx *dctx,
   if (err)
     return 0;
 
-  int32_t func_start_addr = dctx->sfd_funcdesc[func_idx].sfde_func_start_address;
+  const sframe_func_desc_entry *fdep = &dctx->sfd_funcdesc->entry[func_idx];
+  int32_t func_start_addr = fdep->sfde_func_start_address;
 
   return func_start_addr + offsetof_fde_in_sec;
 }
@@ -409,7 +493,7 @@ sframe_fre_check_range_p (sframe_decoder_ctx *dctx, uint32_t func_idx,
   uint32_t pc_offset;
   bool mask_p;
 
-  fdep = &dctx->sfd_funcdesc[func_idx];
+  fdep = &dctx->sfd_funcdesc->entry[func_idx];
   func_start_addr = sframe_decoder_get_secrel_func_start_addr (dctx, func_idx);
   fde_type = sframe_get_fde_type (fdep);
   mask_p = (fde_type == SFRAME_FDE_TYPE_PCMASK);
@@ -428,6 +512,35 @@ sframe_fre_check_range_p (sframe_decoder_ctx *dctx, uint32_t func_idx,
   return (start_ip_offset <= pc_offset) && (end_ip_offset >= pc_offset);
 }
 
+/* Read the on-disk SFrame FDE of SFrame version VER from location BUF of size
+   in bytes equal to BUF_SIZE.
+
+   Return SFRAME_ERR if any error.  If error code is returned, the read values
+   should not be used.  */
+
+static int
+sframe_decode_fde (const char *buf, size_t buf_size, uint8_t ver,
+		   uint32_t *num_fres, uint32_t *fre_type,
+		   uint32_t *fre_offset, size_t *fde_size)
+{
+  if (ver == SFRAME_VERSION_2)
+    {
+      if (buf_size < sizeof (sframe_func_desc_entry_v2))
+	return SFRAME_ERR;
+
+      sframe_func_desc_entry_v2 *fdep = (sframe_func_desc_entry_v2 *) buf;
+      *num_fres = fdep->sfde_func_num_fres;
+      *fre_type = sframe_get_fre_type (fdep);
+      *fre_offset = fdep->sfde_func_start_fre_off;
+
+      *fde_size = sizeof (sframe_func_desc_entry_v2);
+    }
+  else
+    return SFRAME_ERR;
+
+  return 0;
+}
+
 static int
 flip_fre (char *fp, uint32_t fre_type, size_t *fre_size)
 {
@@ -477,18 +590,21 @@ flip_sframe (char *frame_buf, size_t buf_size, uint32_t to_foreign)
 {
   unsigned int i, j, prev_frep_index;
   const sframe_header *ihp;
+  uint8_t ver;
   char *fdes;
+  char *fres;
+  const char *buf_end;
   char *fp = NULL;
-  sframe_func_desc_entry *fdep;
   unsigned int num_fdes = 0;
-  unsigned int num_fres = 0;
+  uint32_t num_fres = 0;
   uint32_t fre_type = 0;
   uint32_t fre_offset = 0;
-  size_t esz = 0;
+  size_t esz = 0, fsz = 0;
   size_t hdrsz = 0;
   int err = 0;
   /* For error checking.  */
-  size_t bytes_flipped = 0;
+  size_t fde_bytes_flipped = 0;
+  size_t fre_bytes_flipped = 0;
 
   /* Header must be in host endianness at this time.  */
   ihp = (sframe_header *)frame_buf;
@@ -500,40 +616,39 @@ flip_sframe (char *frame_buf, size_t buf_size, uint32_t to_foreign)
      FDEs and the first FDE in the buffer.  */
   hdrsz = sframe_get_hdr_size (ihp);
   num_fdes = ihp->sfh_num_fdes;
+  ver = ihp->sfh_preamble.sfp_version;
   fdes = frame_buf + hdrsz + ihp->sfh_fdeoff;
-  fdep = (sframe_func_desc_entry *)fdes;
+  fres = frame_buf + hdrsz + ihp->sfh_freoff;
+  buf_end = frame_buf + buf_size;
 
   j = 0;
   prev_frep_index = 0;
-  for (i = 0; i < num_fdes; fdep++, i++)
+  for (i = 0; i < num_fdes; fdes += fsz, i++)
     {
-      if ((char*)fdep >= (frame_buf + buf_size))
+      if (fdes >= buf_end)
 	goto bad;
 
-      if (to_foreign)
-	{
-	  num_fres = fdep->sfde_func_num_fres;
-	  fre_type = sframe_get_fre_type (fdep);
-	  fre_offset = fdep->sfde_func_start_fre_off;
-	}
+      if (to_foreign && sframe_decode_fde (fdes, buf_end - fdes, ver,
+					   &num_fres, &fre_type, &fre_offset,
+					   &fsz))
+	goto bad;
 
-      flip_fde (fdep);
-      bytes_flipped += sizeof (sframe_func_desc_entry);
+      if (flip_fde (fdes, buf_end - fdes, ver, &fsz))
+	goto bad;
 
-      if (!to_foreign)
-	{
-	  num_fres = fdep->sfde_func_num_fres;
-	  fre_type = sframe_get_fre_type (fdep);
-	  fre_offset = fdep->sfde_func_start_fre_off;
-	}
+      fde_bytes_flipped += fsz;
 
-      fp = frame_buf + hdrsz + ihp->sfh_freoff;
-      fp += fre_offset;
+      if (!to_foreign && sframe_decode_fde (fdes, buf_end - fdes, ver,
+					    &num_fres, &fre_type, &fre_offset,
+					    &fsz))
+	goto bad;
+
+      fp = fres + fre_offset;
       for (; j < prev_frep_index + num_fres; j++)
 	{
 	  if (flip_fre (fp, fre_type, &esz))
 	    goto bad;
-	  bytes_flipped += esz;
+	  fre_bytes_flipped += esz;
 
 	  if (esz == 0 || esz > buf_size)
 	    goto bad;
@@ -541,11 +656,18 @@ flip_sframe (char *frame_buf, size_t buf_size, uint32_t to_foreign)
 	}
       prev_frep_index = j;
     }
-  /* All FDEs and FREs must have been endian flipped by now.  */
-  if ((j != ihp->sfh_num_fres) || (bytes_flipped > (buf_size - hdrsz)))
+
+  /* All FDEs must have been endian flipped by now.  */
+  if (i != num_fdes || fde_bytes_flipped > ihp->sfh_freoff - ihp->sfh_fdeoff)
     goto bad;
+
+  /* All FREs must have been endian flipped by now.  */
+  if (j != ihp->sfh_num_fres || fre_bytes_flipped > ihp->sfh_fre_len)
+    goto bad;
+
   /* Optional trailing section padding.  */
-  for (fp = frame_buf + hdrsz + bytes_flipped; fp < frame_buf + buf_size; fp++)
+  size_t frame_size = hdrsz + ihp->sfh_freoff + fre_bytes_flipped;
+  for (fp = frame_buf + frame_size; fp < frame_buf + buf_size; fp++)
     if (*fp != '\0')
       goto bad;
 
@@ -915,7 +1037,7 @@ sframe_decode (const char *sf_buf, size_t sf_size, int *errp)
   char *frame_buf;
   char *tempbuf = NULL;
 
-  int fidx_size;
+  size_t fidx_size;
   uint32_t fre_bytes;
   int foreign_endian = 0;
 
@@ -953,15 +1075,19 @@ sframe_decode (const char *sf_buf, size_t sf_size, int *errp)
 	return sframe_ret_set_errno (errp, SFRAME_ERR_NOMEM);
       memcpy (tempbuf, sf_buf, sf_size);
 
-      /* Flip the header.  */
-      sframe_header *ihp = (sframe_header *) tempbuf;
-      flip_header (ihp);
+      /* Flip the header first.  */
+      if (flip_header (tempbuf, sfp->sfp_version))
+	{
+	  sframe_ret_set_errno (errp, SFRAME_ERR_BUF_INVAL);
+	  goto decode_fail_free;
+	}
       /* Flip the rest of the SFrame section data buffer.  */
       if (flip_sframe (tempbuf, sf_size, 0))
 	{
-	  free (tempbuf);
-	  return sframe_ret_set_errno (errp, SFRAME_ERR_BUF_INVAL);
+	  sframe_ret_set_errno (errp, SFRAME_ERR_BUF_INVAL);
+	  goto decode_fail_free;
 	}
+
       frame_buf = tempbuf;
       /* This buffer is malloc'd when endian flipping the contents of the input
 	 buffer are needed.  Keep a reference to it so it can be free'd up
@@ -984,17 +1110,21 @@ sframe_decode (const char *sf_buf, size_t sf_size, int *errp)
   frame_buf += hdrsz;
 
   /* Handle the SFrame Function Descriptor Entry section.  */
-  fidx_size = dhp->sfh_num_fdes * sizeof (sframe_func_desc_entry);
-  dctx->sfd_funcdesc = malloc (fidx_size);
-  if (dctx->sfd_funcdesc == NULL)
+  if (sframe_fde_tbl_alloc (&dctx->sfd_funcdesc, dhp->sfh_num_fdes))
     {
       sframe_ret_set_errno (errp, SFRAME_ERR_NOMEM);
       goto decode_fail_free;
     }
-  /* SFrame FDEs are at an offset of sfh_fdeoff from SFrame header end.  */
-  memcpy (dctx->sfd_funcdesc, frame_buf + dhp->sfh_fdeoff, fidx_size);
 
-  debug_printf ("%u total fidx size\n", fidx_size);
+  /* SFrame FDEs are at an offset of sfh_fdeoff from SFrame header end.  */
+  if (sframe_fde_tbl_init (dctx->sfd_funcdesc, frame_buf + dhp->sfh_fdeoff,
+			   &fidx_size, dhp->sfh_num_fdes, sfp->sfp_version))
+    {
+      sframe_ret_set_errno (errp, SFRAME_ERR_BUF_INVAL);
+      goto decode_fail_free;
+    }
+
+  debug_printf ("%zu total fidx size\n", fidx_size);
 
   /* Handle the SFrame Frame Row Entry section.  */
   dctx->sfd_fres = (char *) malloc (dhp->sfh_fre_len);
@@ -1119,7 +1249,7 @@ sframe_get_funcdesc_with_addr_internal (sframe_decoder_ctx *ctx, int32_t addr,
     return sframe_ret_set_errno (errp, SFRAME_ERR_FDE_NOTSORTED);
 
   /* Do the binary search.  */
-  fdp = (sframe_func_desc_entry *) ctx->sfd_funcdesc;
+  fdp = (sframe_func_desc_entry *) ctx->sfd_funcdesc->entry;
   low = 0;
   high = dhp->sfh_num_fdes - 1;
   while (low <= high)
@@ -1364,25 +1494,25 @@ sframe_decoder_get_fre (sframe_decoder_ctx *ctx,
 
 /* SFrame Encoder.  */
 
-/* Get a reference to the ENCODER's SFrame header.  */
+/* Get a reference to the SFrame header, given the encoder context ECTX.  */
 
 static sframe_header *
-sframe_encoder_get_header (sframe_encoder_ctx *encoder)
+sframe_encoder_get_header (sframe_encoder_ctx *ectx)
 {
   sframe_header *hp = NULL;
-  if (encoder)
-    hp = &encoder->sfe_header;
+  if (ectx)
+    hp = &ectx->sfe_header;
   return hp;
 }
 
 static sframe_func_desc_entry *
-sframe_encoder_get_funcdesc_at_index (sframe_encoder_ctx *encoder,
+sframe_encoder_get_funcdesc_at_index (sframe_encoder_ctx *ectx,
 				      uint32_t func_idx)
 {
   sframe_func_desc_entry *fde = NULL;
-  if (func_idx < sframe_encoder_get_num_fidx (encoder))
+  if (func_idx < sframe_encoder_get_num_fidx (ectx))
     {
-      sf_fde_tbl *func_tbl = encoder->sfe_funcdesc;
+      sf_fde_tbl *func_tbl = ectx->sfe_funcdesc;
       fde = func_tbl->entry + func_idx;
     }
   return fde;
@@ -1397,18 +1527,18 @@ sframe_encode (uint8_t ver, uint8_t flags, uint8_t abi_arch,
 	       int8_t fixed_fp_offset, int8_t fixed_ra_offset, int *errp)
 {
   sframe_header *hp;
-  sframe_encoder_ctx *encoder;
+  sframe_encoder_ctx *ectx;
 
   if (ver != SFRAME_VERSION)
     return sframe_ret_set_errno (errp, SFRAME_ERR_VERSION_INVAL);
 
-  if ((encoder = malloc (sizeof (sframe_encoder_ctx))) == NULL)
+  if ((ectx = malloc (sizeof (sframe_encoder_ctx))) == NULL)
     return sframe_ret_set_errno (errp, SFRAME_ERR_NOMEM);
 
-  memset (encoder, 0, sizeof (sframe_encoder_ctx));
+  memset (ectx, 0, sizeof (sframe_encoder_ctx));
 
   /* Get the SFrame header and update it.  */
-  hp = sframe_encoder_get_header (encoder);
+  hp = sframe_encoder_get_header (ectx);
   hp->sfh_preamble.sfp_version = ver;
   hp->sfh_preamble.sfp_magic = SFRAME_MAGIC;
   hp->sfh_preamble.sfp_flags = flags;
@@ -1423,17 +1553,17 @@ sframe_encode (uint8_t ver, uint8_t flags, uint8_t abi_arch,
   hp->sfh_cfa_fixed_fp_offset = fixed_fp_offset;
   hp->sfh_cfa_fixed_ra_offset = fixed_ra_offset;
 
-  return encoder;
+  return ectx;
 }
 
-/* Free the encoder context.  */
+/* Free the encoder context ECTXP.  */
 
 void
-sframe_encoder_free (sframe_encoder_ctx **encoder)
+sframe_encoder_free (sframe_encoder_ctx **ectxp)
 {
-  if (encoder != NULL)
+  if (ectxp != NULL)
     {
-      sframe_encoder_ctx *ectx = *encoder;
+      sframe_encoder_ctx *ectx = *ectxp;
       if (ectx == NULL)
 	return;
 
@@ -1453,58 +1583,58 @@ sframe_encoder_free (sframe_encoder_ctx **encoder)
 	  ectx->sfe_data = NULL;
 	}
 
-      free (*encoder);
-      *encoder = NULL;
+      free (*ectxp);
+      *ectxp = NULL;
     }
 }
 
-/* Get the size of the SFrame header from the encoder ctx ENCODER.  */
+/* Get the size of the SFrame header from the encoder context ECTX.  */
 
 unsigned int
-sframe_encoder_get_hdr_size (sframe_encoder_ctx *encoder)
+sframe_encoder_get_hdr_size (sframe_encoder_ctx *ectx)
 {
-  const sframe_header *ehp = sframe_encoder_get_header (encoder);
+  const sframe_header *ehp = sframe_encoder_get_header (ectx);
   return sframe_get_hdr_size (ehp);
 }
 
-/* Get the abi/arch info from the SFrame encoder context ENCODER.  */
+/* Get the SFrame abi/arch info from the encoder context ECTX.  */
 
 uint8_t
-sframe_encoder_get_abi_arch (sframe_encoder_ctx *encoder)
+sframe_encoder_get_abi_arch (sframe_encoder_ctx *ectx)
 {
   uint8_t abi_arch = 0;
-  const sframe_header *ehp = sframe_encoder_get_header (encoder);
+  const sframe_header *ehp = sframe_encoder_get_header (ectx);
   if (ehp)
     abi_arch = ehp->sfh_abi_arch;
   return abi_arch;
 }
 
-/* Get the format version from the SFrame encoder context ENCODER.  */
+/* Get the SFrame format version from the encoder context ECTX.  */
 
 uint8_t
-sframe_encoder_get_version (sframe_encoder_ctx *encoder)
+sframe_encoder_get_version (sframe_encoder_ctx *ectx)
 {
-  const sframe_header *ehp = sframe_encoder_get_header (encoder);
+  const sframe_header *ehp = sframe_encoder_get_header (ectx);
   return ehp->sfh_preamble.sfp_version;
 }
 
-/* Get the section flags from the SFrame encoder context ENCODER.  */
+/* Get the SFrame flags from the encoder context ECTX.  */
 
 uint8_t
-sframe_encoder_get_flags (sframe_encoder_ctx *encoder)
+sframe_encoder_get_flags (sframe_encoder_ctx *ectx)
 {
-  const sframe_header *ehp = sframe_encoder_get_header (encoder);
+  const sframe_header *ehp = sframe_encoder_get_header (ectx);
   return ehp->sfh_preamble.sfp_flags;
 }
 
-/* Return the number of function descriptor entries in the SFrame encoder
-   ENCODER.  */
+/* Return the number of SFrame function descriptor entries in the encoder
+   context ECTX.  */
 
 uint32_t
-sframe_encoder_get_num_fidx (sframe_encoder_ctx *encoder)
+sframe_encoder_get_num_fidx (sframe_encoder_ctx *ectx)
 {
   uint32_t num_fdes = 0;
-  const sframe_header *ehp = sframe_encoder_get_header (encoder);
+  const sframe_header *ehp = sframe_encoder_get_header (ectx);
   if (ehp)
     num_fdes = ehp->sfh_num_fdes;
   return num_fdes;
@@ -1512,7 +1642,7 @@ sframe_encoder_get_num_fidx (sframe_encoder_ctx *encoder)
 
 /* Get the offset of the sfde_func_start_address field (from the start of the
    on-disk layout of the SFrame section) of the FDE at FUNC_IDX in the encoder
-   context ENCODER.
+   context ECTX.
 
    If FUNC_IDX is more than the number of SFrame FDEs in the section, sets
    error code in ERRP, but returns the (hypothetical) offset.  This is useful
@@ -1520,24 +1650,24 @@ sframe_encoder_get_num_fidx (sframe_encoder_ctx *encoder)
    emitted.  */
 
 uint32_t
-sframe_encoder_get_offsetof_fde_start_addr (sframe_encoder_ctx *encoder,
+sframe_encoder_get_offsetof_fde_start_addr (sframe_encoder_ctx *ectx,
 					    uint32_t func_idx, int *errp)
 {
-  if (func_idx >= sframe_encoder_get_num_fidx (encoder))
+  if (func_idx >= sframe_encoder_get_num_fidx (ectx))
     sframe_ret_set_errno (errp, SFRAME_ERR_FDE_INVAL);
   else if (errp)
     *errp = 0;
 
-  return (sframe_encoder_get_hdr_size (encoder)
+  return (sframe_encoder_get_hdr_size (ectx)
 	  + func_idx * sizeof (sframe_func_desc_entry)
 	  + offsetof (sframe_func_desc_entry, sfde_func_start_address));
 }
 
-/* Add an FRE to function at FUNC_IDX'th function descriptor entry in
-   the encoder context.  */
+/* Add an SFrame FRE to function at FUNC_IDX'th function descriptor entry in
+   the encoder context ECTX.  */
 
 int
-sframe_encoder_add_fre (sframe_encoder_ctx *encoder,
+sframe_encoder_add_fre (sframe_encoder_ctx *ectx,
 			unsigned int func_idx,
 			sframe_frame_row_entry *frep)
 {
@@ -1549,19 +1679,19 @@ sframe_encoder_add_fre (sframe_encoder_ctx *encoder,
   size_t fre_tbl_sz;
   int err = 0;
 
-  if (encoder == NULL || frep == NULL)
+  if (ectx == NULL || frep == NULL)
     return sframe_set_errno (&err, SFRAME_ERR_INVAL);
   if (!sframe_fre_sanity_check_p (frep))
     return sframe_set_errno (&err, SFRAME_ERR_FRE_INVAL);
 
   /* Use func_idx to gather the function descriptor entry.  */
-  fdep = sframe_encoder_get_funcdesc_at_index (encoder, func_idx);
+  fdep = sframe_encoder_get_funcdesc_at_index (ectx, func_idx);
 
   if (fdep == NULL)
     return sframe_set_errno (&err, SFRAME_ERR_FDE_NOTFOUND);
 
   fre_type = sframe_get_fre_type (fdep);
-  sf_fre_tbl *fre_tbl = encoder->sfe_fres;
+  sf_fre_tbl *fre_tbl = ectx->sfe_fres;
 
   if (fre_tbl == NULL)
     {
@@ -1613,10 +1743,10 @@ sframe_encoder_add_fre (sframe_encoder_ctx *encoder,
   esz = sframe_fre_entry_size (frep, fre_type);
   fre_tbl->count++;
 
-  encoder->sfe_fres = fre_tbl;
-  encoder->sfe_fre_nbytes += esz;
+  ectx->sfe_fres = fre_tbl;
+  ectx->sfe_fre_nbytes += esz;
 
-  ehp = sframe_encoder_get_header (encoder);
+  ehp = sframe_encoder_get_header (ectx);
   ehp->sfh_num_fres = fre_tbl->count;
 
   /* Update the value of the number of FREs for the function.  */
@@ -1627,16 +1757,16 @@ sframe_encoder_add_fre (sframe_encoder_ctx *encoder,
 bad:
   if (fre_tbl != NULL)
     free (fre_tbl);
-  encoder->sfe_fres = NULL;
-  encoder->sfe_fre_nbytes = 0;
+  ectx->sfe_fres = NULL;
+  ectx->sfe_fre_nbytes = 0;
   return -1;
 }
 
-/* Add a new function descriptor entry with START_ADDR, FUNC_SIZE and NUM_FRES
-   to the encoder.  */
+/* Add a new SFrame function descriptor entry with START_ADDR, FUNC_SIZE and
+   FUNC_INFO to the encoder context ECTX.  */
 
 int
-sframe_encoder_add_funcdesc (sframe_encoder_ctx *encoder,
+sframe_encoder_add_funcdesc (sframe_encoder_ctx *ectx,
 			     int32_t start_addr,
 			     uint32_t func_size,
 			     unsigned char func_info,
@@ -1648,11 +1778,11 @@ sframe_encoder_add_funcdesc (sframe_encoder_ctx *encoder,
   int err = 0;
 
   /* FIXME book-keep num_fres for error checking.  */
-  if (encoder == NULL)
+  if (ectx == NULL)
     return sframe_set_errno (&err, SFRAME_ERR_INVAL);
 
-  fd_info = encoder->sfe_funcdesc;
-  ehp = sframe_encoder_get_header (encoder);
+  fd_info = ectx->sfe_funcdesc;
+  ehp = sframe_encoder_get_header (ectx);
 
   if (fd_info == NULL)
     {
@@ -1689,7 +1819,7 @@ sframe_encoder_add_funcdesc (sframe_encoder_ctx *encoder,
      sframe_encoder_add_fre.  */
   fd_info->entry[fd_info->count].sfde_func_size = func_size;
   fd_info->entry[fd_info->count].sfde_func_start_fre_off
-    = encoder->sfe_fre_nbytes;
+    = ectx->sfe_fre_nbytes;
 #if 0
   // Linker optimization test code cleanup later ibhagat TODO FIXME
   uint32_t fre_type = sframe_calc_fre_type (func_size);
@@ -1699,25 +1829,24 @@ sframe_encoder_add_funcdesc (sframe_encoder_ctx *encoder,
 #endif
   fd_info->entry[fd_info->count].sfde_func_info = func_info;
   fd_info->count++;
-  encoder->sfe_funcdesc = fd_info;
+  ectx->sfe_funcdesc = fd_info;
   ehp->sfh_num_fdes++;
   return 0;
 
 bad:
   if (fd_info != NULL)
     free (fd_info);
-  encoder->sfe_funcdesc = NULL;
+  ectx->sfe_funcdesc = NULL;
   ehp->sfh_num_fdes = 0;
   return -1;
 }
 
-/* Add a new function descriptor entry with START_ADDR, FUNC_SIZE, FUNC_INFO
-   and REP_BLOCK_SIZE to the encoder.
-
-   This API is valid only for SFrame format version 2.  */
+/* Add a new SFrame function descriptor entry with START_ADDR, FUNC_SIZE,
+   FUNC_INFO and REP_BLOCK_SIZE to the encoder context ECTX.  This API is valid
+   only for SFrame format version 2.  */
 
 int
-sframe_encoder_add_funcdesc_v2 (sframe_encoder_ctx *encoder,
+sframe_encoder_add_funcdesc_v2 (sframe_encoder_ctx *ectx,
 				int32_t start_addr,
 				uint32_t func_size,
 				unsigned char func_info,
@@ -1727,28 +1856,27 @@ sframe_encoder_add_funcdesc_v2 (sframe_encoder_ctx *encoder,
   sf_fde_tbl *fd_info;
   int err;
 
-  if (encoder == NULL
-      || sframe_encoder_get_version (encoder) == SFRAME_VERSION_1)
+  if (ectx == NULL || sframe_encoder_get_version (ectx) == SFRAME_VERSION_1)
     return sframe_set_errno (&err, SFRAME_ERR_INVAL);
 
-  err = sframe_encoder_add_funcdesc (encoder, start_addr, func_size, func_info,
+  err = sframe_encoder_add_funcdesc (ectx, start_addr, func_size, func_info,
 				     num_fres);
   if (err)
     return SFRAME_ERR;
 
-  fd_info = encoder->sfe_funcdesc;
+  fd_info = ectx->sfe_funcdesc;
   fd_info->entry[fd_info->count-1].sfde_func_rep_size = rep_block_size;
 
   return 0;
 }
 
 static int
-sframe_sort_funcdesc (sframe_encoder_ctx *encoder)
+sframe_sort_funcdesc (sframe_encoder_ctx *ectx)
 {
-  sframe_header *ehp = sframe_encoder_get_header (encoder);
+  sframe_header *ehp = sframe_encoder_get_header (ectx);
 
   /* Sort and write out the FDE table.  */
-  sf_fde_tbl *fd_info = encoder->sfe_funcdesc;
+  sf_fde_tbl *fd_info = ectx->sfe_funcdesc;
   if (fd_info)
     {
       /* The new encoding of sfde_func_start_address means the distances are
@@ -1758,14 +1886,14 @@ sframe_sort_funcdesc (sframe_encoder_ctx *encoder)
 
       for (unsigned int i = 0; i < fd_info->count; i++)
 	fd_info->entry[i].sfde_func_start_address
-	  += sframe_encoder_get_offsetof_fde_start_addr (encoder, i, NULL);
+	  += sframe_encoder_get_offsetof_fde_start_addr (ectx, i, NULL);
 
       qsort (fd_info->entry, fd_info->count,
 	     sizeof (sframe_func_desc_entry), fde_func);
 
       for (unsigned int i = 0; i < fd_info->count; i++)
 	fd_info->entry[i].sfde_func_start_address
-	  -= sframe_encoder_get_offsetof_fde_start_addr (encoder, i, NULL);
+	  -= sframe_encoder_get_offsetof_fde_start_addr (ectx, i, NULL);
 
       /* Update preamble's flags.  */
       ehp->sfh_preamble.sfp_flags |= SFRAME_F_FDE_SORTED;
@@ -1855,10 +1983,11 @@ sframe_encoder_write_fre (char *contents, sframe_frame_row_entry *frep,
 }
 
 /* Serialize the core contents of the SFrame section and write out to the
-   output buffer held in the ENCODER.  Return SFRAME_ERR if failure.  */
+   output buffer held in the encoder context ECTX.  Return SFRAME_ERR if
+   failure.  */
 
 static int
-sframe_encoder_write_sframe (sframe_encoder_ctx *encoder)
+sframe_encoder_write_sframe (sframe_encoder_ctx *ectx)
 {
   char *contents;
   size_t buf_size;
@@ -1877,15 +2006,15 @@ sframe_encoder_write_sframe (sframe_encoder_ctx *encoder)
   uint32_t fre_type;
   int err = 0;
 
-  contents = encoder->sfe_data;
-  buf_size = encoder->sfe_data_size;
-  num_fdes = sframe_encoder_get_num_fidx (encoder);
+  contents = ectx->sfe_data;
+  buf_size = ectx->sfe_data_size;
+  num_fdes = sframe_encoder_get_num_fidx (ectx);
   all_fdes_size = num_fdes * sizeof (sframe_func_desc_entry);
-  ehp = sframe_encoder_get_header (encoder);
+  ehp = sframe_encoder_get_header (ectx);
   hdr_size = sframe_get_hdr_size (ehp);
 
-  fd_info = encoder->sfe_funcdesc;
-  fr_info = encoder->sfe_fres;
+  fd_info = ectx->sfe_funcdesc;
+  fr_info = ectx->sfe_fres;
 
   /* Sanity checks:
      - buffers must be malloc'd by the caller.  */
@@ -1935,19 +2064,19 @@ sframe_encoder_write_sframe (sframe_encoder_ctx *encoder)
 
   sframe_assert (fre_size == ehp->sfh_fre_len);
   sframe_assert (global == ehp->sfh_num_fres);
-  sframe_assert ((size_t)(contents - encoder->sfe_data) == buf_size);
+  sframe_assert ((size_t)(contents - ectx->sfe_data) == buf_size);
 
   /* Sort the FDE table */
-  sframe_sort_funcdesc (encoder);
+  sframe_sort_funcdesc (ectx);
 
   /* Sanity checks:
      - the FDE section must have been sorted by now on the start address
      of each function.  */
-  if (!(sframe_encoder_get_flags (encoder) & SFRAME_F_FDE_SORTED)
+  if (!(sframe_encoder_get_flags (ectx) & SFRAME_F_FDE_SORTED)
       || (fd_info == NULL))
     return sframe_set_errno (&err, SFRAME_ERR_FDE_INVAL);
 
-  contents = encoder->sfe_data;
+  contents = ectx->sfe_data;
   /* Write out the SFrame header.  The SFrame header in the encoder
      object has already been updated with correct offsets by the caller.  */
   memcpy (contents, ehp, hdr_size);
@@ -1960,11 +2089,12 @@ sframe_encoder_write_sframe (sframe_encoder_ctx *encoder)
   return 0;
 }
 
-/* Serialize the contents of the encoder and return the buffer.  ENCODED_SIZE
-   is updated to the size of the buffer.  */
+/* Serialize the contents of the encoder context ECTX and return the buffer.
+   ENCODED_SIZE is updated to the size of the buffer.
+   Sets ERRP if failure.  */
 
 char *
-sframe_encoder_write (sframe_encoder_ctx *encoder,
+sframe_encoder_write (sframe_encoder_ctx *ectx,
 		      size_t *encoded_size, int *errp)
 {
   sframe_header *ehp;
@@ -1973,25 +2103,28 @@ sframe_encoder_write (sframe_encoder_ctx *encoder,
 
   /* Initialize the encoded_size to zero.  This makes it simpler to just
      return from the function in case of failure.  Free'ing up of
-     encoder->sfe_data is the responsibility of the caller.  */
+     ectx->sfe_data is the responsibility of the caller.  */
   *encoded_size = 0;
 
-  if (encoder == NULL || encoded_size == NULL || errp == NULL)
+  if (ectx == NULL || encoded_size == NULL || errp == NULL)
     return sframe_ret_set_errno (errp, SFRAME_ERR_INVAL);
 
-  ehp = sframe_encoder_get_header (encoder);
+  ehp = sframe_encoder_get_header (ectx);
   hdrsize = sframe_get_hdr_size (ehp);
-  fsz = sframe_encoder_get_num_fidx (encoder)
-    * sizeof (sframe_func_desc_entry);
-  fresz = encoder->sfe_fre_nbytes;
+  fsz = sframe_encoder_get_num_fidx (ectx) * sizeof (sframe_func_desc_entry);
+  fresz = ectx->sfe_fre_nbytes;
+
+  /* Encoder writes out data in the latest SFrame format version.  */
+  if (sframe_encoder_get_version (ectx) != SFRAME_VERSION)
+    return sframe_ret_set_errno (errp, SFRAME_ERR_VERSION_INVAL);
 
   /* The total size of buffer is the sum of header, SFrame Function Descriptor
      Entries section and the FRE section.  */
   bufsize = hdrsize + fsz + fresz;
-  encoder->sfe_data = (char *) malloc (bufsize);
-  if (encoder->sfe_data == NULL)
+  ectx->sfe_data = (char *) malloc (bufsize);
+  if (ectx->sfe_data == NULL)
     return sframe_ret_set_errno (errp, SFRAME_ERR_NOMEM);
-  encoder->sfe_data_size = bufsize;
+  ectx->sfe_data_size = bufsize;
 
   /* Update the information in the SFrame header.  */
   /* SFrame FDE section follows immediately after the header.  */
@@ -2003,17 +2136,18 @@ sframe_encoder_write (sframe_encoder_ctx *encoder,
   foreign_endian = need_swapping (ehp->sfh_abi_arch);
 
   /* Write out the FDE Index and the FRE table in the sfe_data. */
-  if (sframe_encoder_write_sframe (encoder))
+  if (sframe_encoder_write_sframe (ectx))
     return sframe_ret_set_errno (errp, SFRAME_ERR_BUF_INVAL);
 
   /* Endian flip the contents if necessary.  */
   if (foreign_endian)
     {
-      if (flip_sframe (encoder->sfe_data, bufsize, 1))
+      if (flip_sframe (ectx->sfe_data, bufsize, 1))
+	return sframe_ret_set_errno (errp, SFRAME_ERR_BUF_INVAL);
+      if (flip_header (ectx->sfe_data, SFRAME_VERSION))
 	return sframe_ret_set_errno (errp, SFRAME_ERR_BUF_INVAL);
-      flip_header ((sframe_header*)encoder->sfe_data);
     }
 
   *encoded_size = bufsize;
-  return encoder->sfe_data;
+  return ectx->sfe_data;
 }
